#!/usr/local/bin/perl

use Getopt::Std;
use strict;

############################################################################
my $GlobalInterruptFlag;
my $GlobalForkPID;
############################################################################
sub run {

#  my $dir = '/home/voronkov/CASC/RUN';                # file with the viy program
  my $dir = $ENV{'VAMPIRE_CASC_JC_DIR'};   # file with the viy program
  my $viy = "$dir/bin/viy";            # file with the viy program
  my $vampire = "$dir/bin/vampire";

  # read arguments
  my %args;
  getopts ("p:t:m:",\%args);
  my $timeLimit = $args{'t'};
    die "No -t option" unless ( defined $timeLimit );
  my $problem = $args{'p'};
    die "No -p option" unless ( defined $problem );
    my $name = fileToName ("$problem.p");
  my $memLimit = $args{'m'};
    die "No -m option" unless ( defined $memLimit );

  my $tabFile = "$dir/tab/$name.tab";  # tab file name for Vampire
  my $inFile = "$dir/temp/$name.in";    # input file for Vampire (and output for viy)
  # sequence of parameters produced by viy
  my @params = `$viy -i $problem -t $timeLimit -m $memLimit -T $tabFile -O $inFile`;

  # read Vampire's output
  PARAMETERSETTING:
  foreach my $param (@params) {
    # print $param;
    chomp $param;
    if (!defined($GlobalForkPID = open(Handle,"$vampire $param $inFile|"))) {
        die("ERROR: Cannot fork system process\n");
    }
    my @output = <Handle>;
    close(Handle);
    #print "$vampire $param $inFile\n";
    foreach my $line (@output) {
      print("$line");
      if ( index ($line, 'Proof found') >= 0) {
        print @output;
        unlink $inFile;
        system("/bin/cat $tabFile");
        unlink $tabFile;
        return;
      }
    }
    if ($GlobalInterruptFlag) {
      last PARAMETERSETTING;
    }
  }
  unlink $inFile;
  unlink $tabFile;
  print "Proof not found\n";

}

############################################################################
#   fileToName ($file) -
#     name of problem corresponding to the file, e.g.
#     fileToName ('COL/COL053-1.p') = 'COL053-1'
sub fileToName ($) {
  my $file = shift;
  my ($name) = $file =~
        m/          # find
          ([^\/]*)  # any sequence of characters different from backslash
          \.p       # followed by .p
        /x;         # x is for formatting the pattern
  # remove leading spaces, if any
  $name =~ s/^\s+//;
  return $name;
} # Problem::fileToName

############################################################################
sub  SIGHandler {
    my ($Signal) = @_;

    print("Mamamia, the vurdalak, just got a $Signal\n");
    $GlobalInterruptFlag = 1;
    if ($GlobalForkPID >  0) {
        kill('SIGKILL',$GlobalForkPID);
    }

}
############################################################################
$SIG{'XCPU'} = 'SIGHandler';
$SIG{'ALRM'} = 'SIGHandler';
$GlobalInterruptFlag = 0;
$| = 1;

run ();

